﻿using BPASmartClient.Helper;
using BPASmartClient.IoT;
using BPASmartClient.Message;
using Microsoft.Toolkit.Mvvm.ComponentModel;
using Microsoft.Toolkit.Mvvm.Input;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using System.Windows.Threading;

namespace BPASmartClient.ViewModel
{
    /// <summary>
    /// 日志界面
    /// </summary>
    public class LogViewModel : ObservableObject
    {
        #region 变量
        public DispatcherTimer dispatcherTimer;
        public string ClientId = System.Configuration.ConfigurationManager.AppSettings["ClientId"].ToString();
        private ObservableCollection<LogModel> _LogModels;public ObservableCollection<LogModel> LogDataGrid
        {
            get
            {
                return _LogModels;
            }
            set
            {
                if (_LogModels == value)
                    return;
                _LogModels = value;
                OnPropertyChanged("LogDataGrid");
            }
        }
        private bool _RealTimeModel = true;public bool RealTimeModel
        {
            get
            {
                return _RealTimeModel;
            }
            set
            {
                if (_RealTimeModel == value)
                    return;
                _RealTimeModel = value;
                OnPropertyChanged("RealTimeModel");
            }
        }
        private bool _TimedClear = true;public bool TimedClear
        {
            get
            {
                return _TimedClear;
            }
            set
            {
                if (_TimedClear == value)
                    return;
                _TimedClear = value;
                OnPropertyChanged("TimedClear");
            }
        }
        #endregion

        #region 单一
        private volatile static LogViewModel _Instance;
        public static LogViewModel GetInstance() => _Instance ?? (_Instance = new LogViewModel());
        private LogViewModel()
        {
            Init();
        }
        #endregion

        #region 函数
        /// <summary>
        /// 初始化
        /// </summary>
        public void Init()
        {
            if (LogDataGrid == null) LogDataGrid = new ObservableCollection<LogModel>();
            logHelper.Fun_InitLog(System.AppDomain.CurrentDomain.BaseDirectory);
            //一般日志
            MessageLog.GetInstance.InfoNotify = new Action<string>((s) =>
            {
                System.Windows.Application.Current?.Dispatcher.Invoke((Action)(() =>
                {
                    LogDataGrid.Insert(0, new LogModel { message = s, type = "Info" });
                    logHelper.GetLogConfigInstance().WriteLog(LogLevel.INFO, s);
                }));
            });
            //设备日志
            MessageLog.GetInstance.DeviceProcessLogNotify = new Action<string, string>((id, s) =>
            {
                System.Windows.Application.Current?.Dispatcher.Invoke((Action)(() =>
                {
                    LogDataGrid.Insert(0, new LogModel { message = s, type = "DeviceLog" });
                    logHelper.GetLogConfigInstance().WriteLog(LogLevel.DEBUG, s);
                }));
            });
            //设备告警日志
            MessageLog.GetInstance.DeviceAlarmLogNotify = new Action<string, string>((id, s) =>
            {
                System.Windows.Application.Current?.Dispatcher.Invoke((Action)(() =>
                {
                    LogDataGrid.Insert(0, new LogModel { message = id, type = "DeviceAlarm" });
                    logHelper.GetLogConfigInstance().WriteLog(LogLevel.WARN, id);
                }));
            });
            //错误日志
            MessageLog.GetInstance.ExInfoNotify = new Action<string>((s) =>
            {
                System.Windows.Application.Current?.Dispatcher.Invoke((Action)(() =>
                {
                    LogDataGrid.Insert(0, new LogModel { message = s, type = "Error" });
                    logHelper.GetLogConfigInstance().WriteLog(LogLevel.ERROR, s);
                    DataVClient.GetInstance().HttpAddLog(new LogTable
                    {
                        ClientId = ClientId,
                        LogTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"),
                        LogType = "严重",
                        LogMessage = s,
                        LogVla = "错误日志"
                    });
                }));
            });

            ExcelCommand = new RelayCommand(() =>
            {
                ExcellOrder();
            });

            OpenCommand = new RelayCommand(() =>
            {
                string msg = string.Format("已找到日志文件,是否打开！      \n      1.如选中不打开，则弹出日志文件夹目录。\n      2.打开，则弹出日志文件夹目录，并打开文件。");
                if (System.Windows.MessageBox.Show(msg, "提示", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
                {
                    System.Diagnostics.Process.Start("Explorer", "/select," + logHelper.GetLogConfigInstance().directRollfileAppender.File);
                    logHelper.GetLogConfigInstance().OpenFile(logHelper.GetLogConfigInstance().directRollfileAppender.File);
                }
                else
                    System.Diagnostics.Process.Start("Explorer", "/select," + logHelper.GetLogConfigInstance().directRollfileAppender.File);
            });

            dispatcherTimer = new DispatcherTimer();
            dispatcherTimer.Tick += delegate
            {
                System.Windows.Application.Current?.Dispatcher.Invoke((Action)(() =>
                {
                    if (LogDataGrid.Count > 100)
                    {
                        LogDataGrid.RemoveAt(LogDataGrid.Count - 1);
                    }
                }));

            };
            dispatcherTimer.Interval = TimeSpan.FromSeconds(10);
            dispatcherTimer.Start();
        }
        /// <summary>
        /// 导出数据
        /// </summary>
        public void ExcellOrder()
        {
            if (LogDataGrid.Count > 0)
            {
                string text = "时间    类型    日志内容\n";
                LogDataGrid?.ToList().ForEach(temp =>
                {
                    text = text + temp.time + "    " + temp.type + "    " + temp.message + "\n";
                });
                SaveFileDialog openfile = new SaveFileDialog();
                openfile.Filter = "Txt文件(*.txt)|*.txt";
                if (openfile.ShowDialog() == false)
                {
                    return;
                }
                string path = openfile.FileName;
                if (!System.IO.File.Exists(path))
                {
                    //没有则创建这个文件
                    FileStream fs1 = new FileStream(path, FileMode.Create, FileAccess.Write);//创建写入文件   
                    StreamWriter sw = new StreamWriter(fs1);
                    sw.WriteLine(text);//开始写入值
                    sw.Close();
                    fs1.Close();
                }
                else
                {
                    FileStream fs = new FileStream(path, FileMode.Open, FileAccess.Write);
                    StreamWriter sr = new StreamWriter(fs);
                    sr.WriteLine(text);//开始写入值
                    sr.Close();
                    fs.Close();
                }
                string msg = string.Format("记录导出完成,共导出记录{0}条,是否打开！", LogDataGrid.Count);
                if (System.Windows.MessageBox.Show(msg, "提示", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
                {
                    logHelper.GetLogConfigInstance().OpenFile(openfile.FileName);
                }
            }
            else
                System.Windows.MessageBox.Show("无数据!");

        }
        #endregion

        #region Command
        public RelayCommand ExcelCommand { get; set; }
        public RelayCommand OpenCommand { get; set; }
        #endregion
    }

    public class LogModel : ObservableObject
    {
        public string time { get; set; }
        private string _type;
        public string type
        {
            get
            {
                return _type;
            }
            set
            {
                if (_type == value)
                    return;
                _type = value;
                if (_type == "Error") foreground = new SolidColorBrush((System.Windows.Media.Color)System.Windows.Media.ColorConverter.ConvertFromString("#ed0032"));

                OnPropertyChanged("type");
                OnPropertyChanged("foreground");
            }
        }
        public string message { get; set; }
        private Brush _foreground;
        public Brush foreground
        {
            get
            {
                return _foreground;
            }
            set
            {
                if (_foreground == value)
                    return;
                _foreground = value;
                OnPropertyChanged("foreground");
            }
        }
        public LogModel()
        {
            foreground = new SolidColorBrush((System.Windows.Media.Color)System.Windows.Media.ColorConverter.ConvertFromString("#21bb2e"));
            time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
        }
    }
}
